$.fn.extend({
    tagsy: function(options){
        options = $.extend({
            placeholder: 'Enter a tag',
            data: [],
            inputName: 'tags',
            items: [],
            max: 10,
            allowFreeEntry: true
        }, options);

        var $this = $(this),
            id = 'tagsy-'+(Math.ceil(Math.random()*100)),

            $inputWrapper = $('<div class="input" id='+id+'></div>'),
            $input = $('<input type="text" placeholder="'+options.placeholder+'">'),
            $holder = $('<div/>'),
            $list = $('<ul/>').addClass('list panel panel-default'),

            fields = [];

        function init(){
            $this.addClass('tagsy panel panel-default').click(function(){
                $input.focus();
            });

            $holder.addClass('holder clearfix');

            $input.keyup(function(e){
                var $active = $list.find('li.active');

                $list.find('li').removeClass('active');
                switch(e.which){
                    case 40: // down
                        if($input.val().trim() == '')
                            searchData($(this).val());

                        if($active.length == 0 || $active.is(':last-child'))
                            $active = $list.find('li:first');
                        else
                            $active = $active.next();
                        break;
                    case 38: // up
                        if($active.length == 0 || $active.is(':first-child'))
                            $active = $list.find('li:last');
                        else
                            $active = $active.prev();
                        break;
                    case 27:
                        return closeList();
                        break;

                    default:
                        $active = null;
                        searchData($(this).val());
                        break;
                }

                openList();

                if($active && $active.length){
                    $active.addClass('active');
                    $input.val($active.data('label'));
                    e.preventDefault();
                }

            })

            .keypress(function(e){
                if(e.keyCode == 13){
                    e.stopPropagation();
                    e.preventDefault();

                    if($input.val().trim() != '')
                        addTag($input.val());
                }
            });

            $this.append($holder);
            $holder.append($inputWrapper);
            $inputWrapper.append($input);
            $inputWrapper.append($list);

            resizeInput();
            initItems();
            initData();
        }

        function openList(){
            $list.addClass('open'); return true;
            alert('opened');
        }

        function closeList(){
            $list.find('li').removeClass('active');
            $list.removeClass('open');
            return true;
        }

        function initItems(){
            $.each(options.items, function(i, v){
                addTag(v);
            });
        }

        function findData(search){
            for(var i in options.data) {
                if(options.data[i].value == search || options.data[i].label === search)
                    return options.data[i];
            }
        }

        function initData(){
            $.each(options.data, function(i, d){
                if(typeof d == 'string')
                    options.data[i] = {label: d, value: d};
            });
        }

        function removeTag(value){
            fields[value].span.remove();
            fields[value].input.remove();
            $input.show();
            resizeInput();
        }

        function searchData(label){
            $list.find('li').remove();

            var count = 0;
            for(var i in options.data) {
                if(++count > 5) return;

                var data = options.data[i];

                if(data.label.toLowerCase().indexOf(label) > -1){
                    $item = $('<li data-value="'+data.value+'" data-label="'+data.label+'"></li>')
                        .append('<a href="#">'+data.label+'</a>')
                        .mouseenter(function(e){
                            $input.val($(this).data('label'));
                            e.stopPropagation(); e.preventDefault();
                        })
                        .click(function(e){
                            addTag($(this).data('value'));
                            e.preventDefault();
                        });

                    $list.append($item);
                }
            }
        }

        function addTag(value){
            if(typeof (data = findData(value)) == 'undefined'){
                if(!options.allowFreeEntry)
                    return false;

                data = {label: value, value: value};
            }

            var index = fields.length;

            fields[index] = {
                span: $('<span class="btn btn-sm panel panel-primary">'+data.label+'</span>'),
                input: $('<input type="hidden" name="'+options.inputName+'[]">'),
                label: data.label
            };

            fields[index].span.click(function(){ removeTag(index); }).insertBefore($inputWrapper);

            $this.append(fields[index].input.val(data.value));

            searchData('');

            if($holder.find('.btn').length < options.max || options.max == 0)
                resizeInput();
            else {
                $input.hide();
                closeList();
            }
        }

        function resizeInput(){
            $holder.width($holder.width());
            var holderWidth = $holder.width();

            var availSpace = holderWidth;
            var l = $holder.find('.btn').each(function(){
                var btnWidth = $(this).outerWidth() + 5;
                availSpace -= btnWidth;

                if(availSpace < 0)
                    availSpace = holderWidth - btnWidth;
            }).length;

            $input.outerWidth(availSpace>70? availSpace: holderWidth);
            $input.val('').focus();
        }

        init();
    }
});
